/*
Module : EnumWindow.cpp
Purpose: Implementation for class which implements the IEnumWindow interface
Created: PJN / 20-01-2002

Copyright (c) 2002 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "EnumWindow.h"


//////////////////// Implementation ///////////////////////////////////////////

CEnumWindow::CEnumWindow() noexcept : m_hWnd{nullptr}
{
}

void CEnumWindow::SetHWND(_In_ HWND hWnd) noexcept
{
  m_hWnd = hWnd;
}

#pragma warning(suppress: 26433 26487)
STDMETHODIMP CEnumWindow::get_Title(BSTR* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

  ATL::CAtlString sTitle;
#pragma warning(suppress: 26485)
  GetWindowText(m_hWnd, sTitle.GetBuffer(4096), 4096);
  sTitle.ReleaseBuffer();
#pragma warning(suppress: 26485)
  *pVal = SysAllocString(ATL::CT2W{sTitle});
  return S_OK;
}

#pragma warning(suppress: 26433 26440)
STDMETHODIMP CEnumWindow::get_HWND(LONGLONG* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

#pragma warning(suppress: 26490)
  *pVal = reinterpret_cast<LONGLONG>(m_hWnd);

  return S_OK;
}

#pragma warning(suppress: 26433 26440)
STDMETHODIMP CEnumWindow::get_Width(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

  RECT r{};
  if (GetWindowRect(m_hWnd, &r))
  {
    *pVal = r.right - r.left;
    return S_OK;
  }
  else
    return ATL::AtlHresultFromLastError();
}

#pragma warning(suppress: 26433 26440)
STDMETHODIMP CEnumWindow::get_Height(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

  RECT r{};
  if (GetWindowRect(m_hWnd, &r))
  {
    *pVal = r.bottom - r.top;
    return S_OK;
  }
  else
    return ATL::AtlHresultFromLastError();
}

#pragma warning(suppress: 26433 26440)
STDMETHODIMP CEnumWindow::get_X(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

  RECT r{};
  if (GetWindowRect(m_hWnd, &r))
  {
    *pVal = r.left;
    return S_OK;
  }
  else
    return ATL::AtlHresultFromLastError();
}

#pragma warning(suppress: 26433 26440)
STDMETHODIMP CEnumWindow::get_Y(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

  RECT r{};
  if (GetWindowRect(m_hWnd, &r))
  {
    *pVal = r.top;
    return S_OK;
  }
  else
    return ATL::AtlHresultFromLastError();
}

#pragma warning(suppress: 26433 26440)
STDMETHODIMP CEnumWindow::get_Style(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

#pragma warning(suppress: 26472)
  *pVal = static_cast<long>(GetWindowLongPtr(m_hWnd, GWL_STYLE));

  return S_OK;
}

#pragma warning(suppress: 26433 26440)
STDMETHODIMP CEnumWindow::get_ExtendedStyle(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

#pragma warning(suppress: 26472)
  *pVal = static_cast<long>(GetWindowLongPtr(m_hWnd, GWL_EXSTYLE));

  return S_OK;
}

#pragma warning(suppress: 26433 26440 26487)
STDMETHODIMP CEnumWindow::get_Class(BSTR* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

  ATL::CAtlString sClass;
#pragma warning(suppress: 26485)
  const int nGCN{GetClassName(m_hWnd, sClass.GetBuffer(4096), 4096)};
  sClass.ReleaseBuffer();
  if (nGCN == 0)
    return ATL::AtlHresultFromLastError();
#pragma warning(suppress: 26485)
  *pVal = SysAllocString(ATL::CT2W{sClass});
  return S_OK;
}

#pragma warning(suppress: 26433 26440)
STDMETHODIMP CEnumWindow::get_ProcessID(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

#pragma warning(suppress: 26490)
  GetWindowThreadProcessId(m_hWnd, reinterpret_cast<LPDWORD>(pVal));
  return S_OK;
}

#pragma warning(suppress: 26433 26440)
STDMETHODIMP CEnumWindow::get_ThreadID(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

  *pVal = GetWindowThreadProcessId(m_hWnd, nullptr);
  return S_OK;
}
