/*
Module : EnumWindow.h
Purpose: Interface for the class which implements the IEnumWindow interface
Created: PJN / 20-01-2002

Copyright (c) 2002 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once


//////////////////// Includes /////////////////////////////////////////////////

#include "resource.h"
#pragma warning(push)
#pragma warning(disable: 28251)
#include "EnumWnd_i.h"
#pragma warning(pop)


//////////////////// Classes //////////////////////////////////////////////////

class ATL_NO_VTABLE CEnumWindow : public ATL::CComObjectRootEx<ATL::CComSingleThreadModel>,
                                  public ATL::CComCoClass<CEnumWindow, &CLSID_EnumWindow>,
                                  public ATL::IDispatchImpl<IEnumWindow, &__uuidof(IEnumWindow), &LIBID_ENUMWNDLib, 1, 0>
{
public:
//Constructors / Destructors
  CEnumWindow() noexcept;
  CEnumWindow(const CEnumWindow&) = delete;
  CEnumWindow(CEnumWindow&&) = delete;
  virtual ~CEnumWindow() = default;

#pragma warning(suppress: 26440)
DECLARE_REGISTRY_RESOURCEID(IDR_ENUMWINDOW)
DECLARE_NOT_AGGREGATABLE(CEnumWindow)

#pragma warning(suppress: 26434)
DECLARE_PROTECT_FINAL_CONSTRUCT()

#pragma warning(suppress: 26429 26440 26477 26481)
BEGIN_COM_MAP(CEnumWindow) //NOLINT(modernize-avoid-c-arrays, modernize-use-noexcept)
  COM_INTERFACE_ENTRY(IEnumWindow)
  COM_INTERFACE_ENTRY(IDispatch)
#pragma warning(suppress: 26477 26485)
END_COM_MAP()

//Non COM Methods
  CEnumWindow& operator=(const CEnumWindow&) = delete;
  CEnumWindow& operator=(CEnumWindow&&) = delete;
  void SetHWND(_In_ HWND hWnd) noexcept;

//IEnumWindow
  STDMETHOD(get_Title)(BSTR* pVal);
  STDMETHOD(get_HWND)(LONGLONG* pVal);
  STDMETHOD(get_Width)(long* pVal);
  STDMETHOD(get_Height)(long* pVal);
  STDMETHOD(get_X)(long* pVal);
  STDMETHOD(get_Y)(long* pVal);
  STDMETHOD(get_Style)(long* pVal);
  STDMETHOD(get_ExtendedStyle)(long* pVal);
  STDMETHOD(get_Class)(BSTR* pVal);
  STDMETHOD(get_ProcessID)(long* pVal);
  STDMETHOD(get_ThreadID)(long* pVal);

protected:
  HWND m_hWnd; //The HWND which this COM class represents
};

#pragma warning(suppress: 26477)
OBJECT_ENTRY_NON_CREATEABLE_EX_AUTO(__uuidof(EnumWindow), CEnumWindow)
