/*
Module : TopLevelWindows.cpp
Purpose: Implementation for class which implements the ITopLevelWindows interface
Created: PJN / 20-01-2002

Copyright (c) 2002 - 2022 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "TopLevelWindows.h"


//////////////////// Macros / Defines /////////////////////////////////////////

using CComEnumVariant = ATL::CComEnum<IEnumVARIANT, &IID_IEnumVARIANT, VARIANT, ATL::_Copy<VARIANT>>;


//////////////////// Implementation ///////////////////////////////////////////

#pragma warning(suppress: 26433 26487)
STDMETHODIMP CTopLevelWindows::Item(long nIndex, BSTR* pbstrTitle) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pbstrTitle == nullptr)
    return E_POINTER;

  //Validate the index
  if ((nIndex >= 0) && (nIndex < m_hWnds.GetSize()))
  {
    //Get the text and return it
    ATL::CAtlString sTitle;
#pragma warning(suppress: 26485)
    GetWindowText(m_hWnds[nIndex], sTitle.GetBuffer(4096), 4096);
    sTitle.ReleaseBuffer();
#pragma warning(suppress: 26485)
    *pbstrTitle = SysAllocString(ATL::CT2W{sTitle});
    return S_OK;
  }
  else
    return E_INVALIDARG;
}

#pragma warning(suppress: 26433)
STDMETHODIMP CTopLevelWindows::get_Count(long* pVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (pVal == nullptr)
    return E_POINTER;

  //Just return the value
  *pVal = m_hWnds.GetSize();

  return S_OK;
}

#pragma warning(suppress: 26433 26487)
STDMETHODIMP CTopLevelWindows::get__NewEnum(IUnknown** ppVal) //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Validate our parameters
  if (ppVal == nullptr)
    return E_POINTER;

  //Set the Enumerator to nullptr
  *ppVal = nullptr;

#pragma warning(suppress: 26429)
  ATL::CComObject<CComEnumVariant>* pEnumerator{nullptr};
  HRESULT hr{ATL::CComObject<CComEnumVariant>::CreateInstance(&pEnumerator)};
  if (FAILED(hr))
    return hr;

  //Allocate a VARIANT array which will be taken over by the enumerator
  int nWnds{m_hWnds.GetSize()};
  VARIANT* pVarData{nullptr};
#pragma warning(suppress: 26400 26409)
  ATLTRY(pVarData = new VARIANT[nWnds]);
  if (pVarData == nullptr)
  {
    delete pEnumerator;
    return E_OUTOFMEMORY;
  }

  //Copy each item into the VARIANT array
  for (int i{0}; i<nWnds; i++)
  {
    //Initialize each variant
#pragma warning(suppress: 26481)
    VariantInit(&pVarData[i]);

    //Fill in each variant's BSTR
#pragma warning(suppress: 26481)
    pVarData[i].vt = VT_BSTR;
#pragma warning(suppress: 26481)
    hr = Item(i, &pVarData[i].bstrVal);
    if (FAILED(hr))
    {
#pragma warning(suppress: 26401 26409)
      delete [] pVarData;
      delete pEnumerator;
      return hr;
    }
  }

  //Initialize the enumerator
#pragma warning(suppress: 26477 26481)
  hr = pEnumerator->Init(&pVarData[0], &pVarData[nWnds], nullptr, ATL::AtlFlagTakeOwnership);
  if (FAILED(hr))
  {
#pragma warning(suppress: 26401 26409)
    delete [] pVarData;
    delete pEnumerator;
    return hr;
  }
#pragma warning(suppress: 26490)
  hr = pEnumerator->QueryInterface(IID_IUnknown, reinterpret_cast<void**>(ppVal));
  if (FAILED(hr))
  {
    delete pEnumerator;
    return hr;
  }

  return hr;
}

BOOL CALLBACK CTopLevelWindows::_EnumWindows(_In_ HWND hWnd, _In_ LPARAM lParam)
{
  //Convert from the SDK world to the C++ world
#pragma warning(suppress: 26490)
  auto pThis{reinterpret_cast<CTopLevelWindows*>(lParam)};
#pragma warning(suppress: 26477)
  ATLASSUME(pThis != nullptr);

#pragma warning(suppress: 26486)
  return pThis->EnumWindows(hWnd);
}

BOOL CTopLevelWindows::EnumWindows(_In_ HWND hWnd)
{
  //And hive away the hwnd
  return m_hWnds.Add(hWnd);
}

#pragma warning(suppress: 26433)
STDMETHODIMP CTopLevelWindows::Enumerate() //NOLINT(clang-diagnostic-microsoft-exception-spec)
{
  //Free up the currently enumerated windows
  m_hWnds.RemoveAll();

  //Do the enumeration
#pragma warning(suppress: 26490)
  ::EnumWindows(_EnumWindows, reinterpret_cast<LPARAM>(this));

  return S_OK;
}
